#!/bin/sh -e
#
# This script is used by abuild to generate install scripts for managing systemd
# units. It is used by the default_systemd implementation in abuild to automatically
# preset services, disable them prior to uninstall, and reload or restart them
# on upgrade.
#
# systemd.preset reference:
# 	https://www.freedesktop.org/software/systemd/man/latest/systemd.preset.html
# Systemd macro file for RPM:
# 	https://github.com/systemd/systemd/blob/main/src/rpm/macros.systemd.in
# 	https://github.com/systemd/systemd/blob/main/src/rpm/systemd-update-helper.in

# One of "post-install", "pre-deinstall", "post-upgrade"
script="$1"

msg() {
	fmt="[systemd.install] $1"
	shift
	printf "$fmt" $@ >&2
}

# Emit the shebang and common configuration
emit_preamble() {
	cat <<- EOF
	#!/bin/sh -e
	#
	# This install script was generated by abuild for managing systemd units
	#

	SYSTEMD_BUS_TIMEOUT=15s

	EOF
}

case "$script" in
	post-install)
		if [ -z "${UNITS_USER}${UNITS_SYSTEM}" ]; then
			msg 'No units to preset, $UNITS_USER and $UNITS_SYSTEM unset!\n'
			exit 2
		fi

		emit_preamble

		# Emit code to preset user and system services
		if [ -n "$UNITS_SYSTEM" ]; then
			echo "systemctl --no-reload preset $UNITS_SYSTEM || true"
		fi

		if [ -n "$UNITS_USER" ]; then
			echo "systemctl --no-reload preset --global $UNITS_USER"
		fi
		;;
	pre-deinstall)
		if [ -z "${UNITS_USER}${UNITS_SYSTEM}" ]; then
			msg 'No units to preset, $UNITS_USER and $UNITS_SYSTEM unset!\n'
			exit 2
		fi

		emit_preamble

		if [ -n "$UNITS_SYSTEM" ]; then
			echo "units_system=\"$UNITS_SYSTEM\""
			cat <<- 'EOF'

			# Disable units to clean up any symlinks
		    systemctl --no-reload disable --no-warn $units_system

			EOF
		fi

		if [ -n "$UNITS_USER" ]; then
			echo "units_user=\"$UNITS_USER\""
			cat <<- 'EOF'

			# Disable globally (symlinks in /etc/systemd/user)
			systemctl --global disable --no-warn $units_user

			# If on a running system, we also enter each running session and ensure
			# the service is disabled for each user (symlinks in ~/.config/systemd)
			if [ -d "/run/systemd/system" ]; then
			    users=$(systemctl list-units 'user@*' --legend=no | sed -n -r 's/.*user@([0-9]+).service.*/\1/p')
			    for user in $users; do
			        systemctl --user -M "$user@" disable --no-warn $units_user &
			    done
			    wait
			fi
			EOF
		fi
		;;
	post-upgrade)
		# For post-upgrade we have lists of units to reload and restart
		if [ -z "${UNITS_USER_RELOAD}${UNITS_SYSTEM_RELOAD}${UNITS_USER_RESTART}${UNITS_SYSTEM_RESTART}" ]; then
			msg 'No units to reload/restart set in the environment\n'
			exit 2
		fi

		emit_preamble

		# Common post-upgrade preamble
		cat <<- 'EOF'
		# If systemd isn't running then no services are available to reload/restart, so just quit
		if ! [ -d "/run/systemd/system" ]; then
		    exit 0
		fi

		# List of active user sessions
		users=$(systemctl list-units 'user@*' --legend=no | sed -n -r 's/.*user@([0-9]+).service.*/\1/p')

		EOF

		# Mark system services for reload and restart
		if [ -n "$UNITS_SYSTEM_RELOAD" ] || [ -n "$UNITS_SYSTEM_RESTART" ]; then
			echo "units_system_reload=\"$UNITS_SYSTEM_RELOAD\""
			echo "units_system_restart=\"$UNITS_SYSTEM_RESTART\""
			cat <<- 'EOF'
			for unit in $units_system_reload; do
			    systemctl set-property "$unit" Markers=+needs-reload &
			done
			for unit in $units_system_restart; do
			    systemctl set-property "$unit" Markers=+needs-restart &
			done

			EOF
		fi

		# Mark user units for reload and restart
		if [ -n "$UNITS_USER_RELOAD" ] || [ -n "$UNITS_USER_RESTART" ]; then
			echo "units_user_reload=\"$UNITS_USER_RELOAD\""
			echo "units_user_restart=\"$UNITS_USER_RESTART\""
			cat <<- 'EOF'
			for user in $users; do
			    for unit in $units_user_reload; do
			        systemctl --user -M "$user@" set-property "$unit" Markers=+needs-reload &
			    done
			    for unit in $units_user_restart; do
			        systemctl --user -M "$user@" set-property "$unit" Markers=+needs-restart &
			    done
			done
			EOF
		fi

		# Wait for everything to reload/restart
		echo "wait"
		;;
	*)
		msg 'Unknown script to generate: %s\n' "$script"
		exit 2
		;;
esac
